`timescale 1ns / 1ps

module main
(
	input i_clk,
	input i_reset,
	
	output o_bg_buf_shift_en,
	output o_bg_fetch_go,
	output o_ire_enable,
	output o_spr_eval_reset,
	output o_spr_buf_enable,
	output o_spr_fetch_go,
	output o_memcnt_loadall,
	output o_memcnt_loadh,
	output o_memcnt_incv,
	output o_pixel_valid,
	
	output [8:0] o_cur_x,
	output [7:0] o_cur_y,
	
	output o_set_vblank,
	output o_clear_vblank,
	input i_object_clip,
	input i_object_enable,
	input i_pf_clip,
	input i_pf_enable
);

reg [8:0] x;
reg [8:0] y;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		x <= 9'd0;
		y <= 9'd0;
	end
	else begin
		if (x == 9'd340) begin
			x <= 'd0;
			if (y == 9'd261) begin
				y <= 9'd0;
			end
			else begin
				y <= y + 9'd1;
			end
		end
		else begin
			x <= x + 9'd1;
		end
	end
end

wire visible_y = (y < 9'd241);
wire visible_x = (x < 9'd256);

wire x_ens = visible_y && i_object_enable;
wire x_enb = visible_y && i_pf_enable;

wire pixbuffers_enable = (x >= 9'd333 || visible_x);

assign o_bg_buf_shift_en = x_enb && pixbuffers_enable;
assign o_bg_fetch_go = x_enb && (x >= 9'd324 || x <= 9'd247);
assign o_ire_enable = x_ens && visible_x;
assign o_spr_eval_reset = x_ens && (x ==  9'd0);
assign o_spr_buf_enable = x_ens && pixbuffers_enable;
assign o_spr_fetch_go = x_ens && (x ==  9'd257);
assign o_memcnt_loadall = (y == 9'd0) && (x == 9'd257) && (x_ens || x_enb);
assign o_memcnt_loadh = (x == 9'd257) && (x_ens || x_enb) && (y >= 9'd0);
assign o_memcnt_incv = (x == 9'd257) && (x_ens || x_enb) && (y >= 9'd0);

assign o_set_vblank = (y == 9'd241) && (x == 9'd340);
assign o_clear_vblank = (y == 9'd261) && (x == 9'd340);

assign o_pixel_valid = visible_x && visible_y && (y > 9'd0);

assign o_cur_y = y - 9'd1;
assign o_cur_x = x;

endmodule
