`timescale 1ns / 1ps

module nes_standalone
(
	input								MGT_CLK_N,
	input								MGT_CLK_P,
	input 							PROC_RESET_Z,
	input [4:0]						button, //Enter up down left right
	
	output 							VGA_VSYNCH,
	output 							VGA_HSYNCH,
	output 							VGA_OUT_BLANK_Z,
	output 							VGA_COMP_SYNCH,
	output 							VGA_OUT_PIXEL_CLOCK,
	
	output [7:0] 					VGA_OUT_RED,
	output [7:0] 					VGA_OUT_GREEN,
	output [7:0] 					VGA_OUT_BLUE
);

 //A  B   SELECT   START   UP    DOWN   LEFT  RIGHT
wire cont1_data;

//wire PROC_RESET_Z;
//assign PROC_RESET_Z = 1'b1;

wire reset = ~PROC_RESET_Z;

wire clk1;
wire clk5;
wire clk25;

//Working ram on the NES
reg [7:0] wram [2048:0];

reg [7:0] wram_out;
reg wram_wr;

wire nmi;
wire cpu_stall;
wire R_W_n;

//dma signals
wire [23:0] cpu_addr_cpu;
wire [7:0] data_from_cpu_dma, data_to_cpu_dma;
wire R_W_n_cpu;

wire [7:0] ppu_read_data;
reg ppu_rd, ppu_wr;
wire [5:0] ppu_pixel;
wire ppu_pixel_valid;
wire [7:0] ppu_pixel_x;
wire [7:0] ppu_pixel_y;

wire [15:0] cpu_addr;
wire [7:0] data_from_cpu;
reg [7:0] data_to_cpu;

wire [7:0] cart_data_out;
reg cart_wr;

NESclks clkgen
(
	.clk75_n(MGT_CLK_N),
	.clk75_p(MGT_CLK_P),
	.clk1(clk1),
	.clk25(clk25),
	.clk5(clk5)
);


controller_iface cont1 (
	.i_clk(clk1),
	 .i_reset(reset),
	
	 .i_addr(cpu_addr),
	 .i_wr_data(data_from_cpu),
	 .o_rd_data(cont1_data),
	 .i_write_n(R_W_n),
	
	 .i_a(~button[0]),
	 .i_b(1'b0),
	 .i_start(1'b0),
	 .i_select(1'b0),
	 .i_up(~button[1]),
	 .i_down(~button[2]),
	 .i_left(~button[3]),
	 .i_right(~button[4])
);



always @(posedge clk5) begin //or cpu_addr[15:0] can't be included?
	casex(cpu_addr[15:0])
		//PPU
		16'h3xxx,16'h2xxx: begin
								data_to_cpu <= ppu_read_data;
								wram_wr <= 0;
								cart_wr <= 0;
								ppu_wr <= ~R_W_n;
								ppu_rd <= R_W_n;
								end
		
		//WRAM
		16'h0xxx: begin
					data_to_cpu <= wram_out;
					wram_wr <= ~R_W_n;
					cart_wr <= 0;
					ppu_wr <= 0;
					ppu_rd <= 0;
					end
		
		
		//APU - not supported
		//16'h4000-16'h4013
		
		//DMA registers
		//16'h4014,16'h4015h:;
		
		//Controllers
		16'h4016: begin //16'h4017h:;
						data_to_cpu <=cont1_data;
						wram_wr <= 0;
						cart_wr <= 0;
						ppu_wr <= 0;
						ppu_rd <= 0;
					end

		
		//Cartridge Expansion Area
		//16'h41xx,16'h42xx,...,16'h5xxx:;
		
		//Cartridge  CHR RAM
		16'h6xxx,16'h7xxx: begin
								data_to_cpu <= cart_data_out;
								wram_wr <= 0;
								cart_wr <= ~R_W_n;
								ppu_wr <= 0;
								ppu_rd <= 0;
								end
		
		
		//Cartridge PRG-ROM
		16'b1xxxxxxxxxxxxxxx: begin
									data_to_cpu <= cart_data_out; 
									wram_wr <= 0;
									cart_wr <= 0;
									ppu_wr <= 0;
									ppu_rd <= 0;
									end
		
		default: begin
					data_to_cpu <= 8'b0;
					wram_wr <= 0;
					cart_wr <= 0;
					ppu_wr <= 0;
					ppu_rd <= 0;
					end
	endcase
end


ppu the_ppu
(
	.i_clk_54(clk5),
	.i_clk_17(clk1),
	.i_reset(reset),
	
	.i_addr(cpu_addr[2:0]),
	.i_wr_data(data_from_cpu),
	.o_rd_data(ppu_read_data),
	.i_read(ppu_rd),
	.i_write(ppu_wr),
	
	.o_nmi(nmi),
	
	.o_pixel(ppu_pixel),
	.o_pixel_valid(ppu_pixel_valid),
	.o_pixel_x(ppu_pixel_x),
	.o_pixel_y(ppu_pixel_y)
);

vid_out vid_out_inst
(
	.clk25(clk25),
	.clk5(clk5),
	
	.pixel_colour(ppu_pixel),
	.colour_emphasis(2'b0),
	.reset(reset),
	.write_pixel(ppu_pixel_valid),
	.pixel_x(ppu_pixel_x),
	.pixel_y(ppu_pixel_y),
	
	.VGA_VSYNCH(VGA_VSYNCH),
	.VGA_HSYNCH(VGA_HSYNCH),
	.VGA_OUT_BLANK_Z(VGA_OUT_BLANK_Z),
	.VGA_COMP_SYNCH(VGA_COMP_SYNCH),
	.VGA_OUT_PIXEL_CLOCK(VGA_OUT_PIXEL_CLOCK),
	
	.VGA_OUT_RED(VGA_OUT_RED),
	.VGA_OUT_GREEN(VGA_OUT_GREEN),
	.VGA_OUT_BLUE(VGA_OUT_BLUE)
);


//CPU
T65 CPU_6502 (
		.Mode(2'b00), //"00" => 6502, "01" => 65C02, "10" => 65C816
		.Res_n(PROC_RESET_Z),
		.Enable(1'b1),
		.Clk(clk1), 
		.Rdy(~cpu_stall), //Can use to pause the CPU
		.Abort_n(1'b1), 
		.IRQ_n(1'b1), 
		.NMI_n(~nmi), 
		.SO_n(1'b1), 
		.R_W_n(R_W_n_cpu), // No idea what any of these signals do...
		.Sync(), 
		.EF(), 
		.MF(), 
		.XF(), 
		.ML_n(), 
		.VP_n(), 
		.VDA(), 
		.VPA(), // ^^^^
		.A(cpu_addr_cpu), 
		.DI(data_to_cpu_dma), 
		.DO(data_from_cpu_dma)
);

spr_dma spr_dma_inst (
    .i_clk(clk1), 
    .i_reset(reset), 
    .i_addr(cpu_addr_cpu[15:0]), 
    .i_wr_data(data_from_cpu_dma), 
    .o_rd_data(data_to_cpu_dma), 
    .i_write_n(R_W_n_cpu),
	 
    .o_addr(cpu_addr), 
    .o_wr_data(data_from_cpu), 
    .i_rd_data(data_to_cpu), 
    .o_write_n(R_W_n), 
    .o_stall_cpu(cpu_stall)
);

//2K of working RAM

always @ (posedge clk5) begin
	if (wram_wr) begin
		wram[cpu_addr[10:0]] <= data_from_cpu;
		wram_out <= data_from_cpu;
	end else
		wram_out <= wram[cpu_addr[10:0]];
		
end

cartridge cart (
    .o_cpu_data(cart_data_out), 
    .o_ppu_data(),
	 
    .i_cpu_data(data_from_cpu), 
    .i_ppu_data(8'b0),
	 
    .i_cpu_addr(cpu_addr[15:0]), 
    .i_ppu_addr(14'b0),
	 .i_cpu_wr(cart_wr),
	 .i_ppu_wr(1'b0),
	 .clk(clk5)
);

endmodule
