`timescale 1ns / 1ps

module ppu_tb_v;

	// Inputs
	reg OPB_Clk;
	reg OPB_Rst;
	reg [31:0] OPB_ABus;
	reg [3:0] OPB_BE;
	reg [31:0] OPB_DBus;
	reg OPB_RNW;
	reg OPB_select;
	reg OPB_seqAddr;
	reg clk75_n;
	reg clk75_p;

	// Outputs
	wire [31:0] Sl_DBus;
	wire Sl_errAck;
	wire Sl_retry;
	wire Sl_toutSup;
	wire Sl_xferAck;
	wire IP2INTC_Irpt;
	wire VGA_VSYNCH;
	wire VGA_HSYNCH;
	wire VGA_OUT_BLANK_Z;
	wire VGA_COMP_SYNCH;
	wire VGA_OUT_PIXEL_CLOCK;
	wire [7:0] VGA_OUT_RED;
	wire [7:0] VGA_OUT_GREEN;
	wire [7:0] VGA_OUT_BLUE;

	always #5 OPB_Clk = ~OPB_Clk;
	always #6.666 clk75_n = ~clk75_n;
	always #6.666 clk75_p = ~clk75_p;

	task read_ppu;
		input [2:0] addr;
		begin
		OPB_select = 1'b1;
		OPB_ABus = 32'h80000000 + addr;
		OPB_RNW = 1'b1;
		wait(Sl_xferAck == 1'b1);
		@(posedge OPB_Clk);
		OPB_select = 1'b0;
		OPB_ABus = 32'h0;
		end
	endtask
	
	task write_ppu;
		input [2:0] addr;
		input [7:0] data;
		begin
		OPB_select = 1'b1;
		OPB_ABus = 32'h80000000 + addr;
		OPB_RNW = 1'b0;
		OPB_DBus = data;
		wait(Sl_xferAck == 1'b1);
		@(posedge OPB_Clk);
		OPB_select = 1'b0;
		OPB_ABus = 32'h0;
		OPB_DBus = 32'h0;
		end
	endtask
	
		

		

	// Instantiate the Unit Under Test (UUT)
	ppubridge uut (
		.OPB_Clk(OPB_Clk), 
		.OPB_Rst(OPB_Rst), 
		.Sl_DBus(Sl_DBus), 
		.Sl_errAck(Sl_errAck), 
		.Sl_retry(Sl_retry), 
		.Sl_toutSup(Sl_toutSup), 
		.Sl_xferAck(Sl_xferAck), 
		.OPB_ABus(OPB_ABus), 
		.OPB_BE(OPB_BE), 
		.OPB_DBus(OPB_DBus), 
		.OPB_RNW(OPB_RNW), 
		.OPB_select(OPB_select), 
		.OPB_seqAddr(OPB_seqAddr), 
		.IP2INTC_Irpt(IP2INTC_Irpt), 
		.clk75_n(clk75_n), 
		.clk75_p(clk75_p), 
		.VGA_VSYNCH(VGA_VSYNCH), 
		.VGA_HSYNCH(VGA_HSYNCH), 
		.VGA_OUT_BLANK_Z(VGA_OUT_BLANK_Z), 
		.VGA_COMP_SYNCH(VGA_COMP_SYNCH), 
		.VGA_OUT_PIXEL_CLOCK(VGA_OUT_PIXEL_CLOCK), 
		.VGA_OUT_RED(VGA_OUT_RED), 
		.VGA_OUT_GREEN(VGA_OUT_GREEN), 
		.VGA_OUT_BLUE(VGA_OUT_BLUE)
	);

   integer fp;

	initial begin
		// Initialize Inputs
		OPB_Clk = 0;
		OPB_Rst = 1;
		OPB_ABus = 0;
		OPB_BE = 0;
		OPB_DBus = 0;
		OPB_RNW = 0;
		OPB_select = 0;
		OPB_seqAddr = 0;
		clk75_n = 1;
		clk75_p = 0;

      
		// Wait 100 ns for global reset to finish
		#2000;
		OPB_Rst = 0;
		@(posedge OPB_Clk);
		@(posedge OPB_Clk);
		#1;
        
		// Add stimulus here
		//write_ppu(3'd1, 8'd24);

	end
      
endmodule

