module spr_dma
(
	input i_clk,
	input i_reset,
	
	input [15:0] i_addr,
	input [7:0] i_wr_data,
	output reg [7:0] o_rd_data,
	input i_write_n,
	
	output reg [15:0] o_addr,
	output reg [7:0] o_wr_data,
	input [7:0] i_rd_data,
	output reg o_write_n,
	
	output reg o_stall_cpu
);

reg [15:0] dma_addr;
reg [7:0] dma_data;

reg [2:0] state, nextstate;
localparam S_PASSTHRU = 2'd0, S_DMA_READ = 2'd1, S_DMA_WRITE = 2'd2;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		state <= S_PASSTHRU;
	end
	else begin
		state <= nextstate;
		
		case (state)
			S_DMA_READ: begin
				dma_data <= i_rd_data;
			end
		endcase
	end
end

reg [7:0] counter;
reg counter_reset;
reg counter_enable;

always @ (posedge i_clk or posedge i_reset) begin
	if (i_reset) begin
		counter <= 8'd0;
	end
	else begin
		if (counter_reset) begin
			counter <= 8'd255;
			dma_addr <= {i_wr_data, 8'b0};
		end
		else if (counter_enable) begin
			counter <= counter - 8'd1;
			dma_addr <= dma_addr + 16'd1;
		end
	end
end
		

always @* begin
	nextstate = state;
	o_stall_cpu = 1'b0;
	counter_reset = 1'b0;
	counter_enable = 1'b0;
	
	o_addr = 16'bx;
	o_rd_data = 8'bx;
	o_write_n = 1'b1;
	o_wr_data = 8'bx;
	
	case (state)
		S_PASSTHRU: begin
			o_addr = i_addr;
			o_rd_data = i_rd_data;
			o_wr_data = i_wr_data;
			o_write_n = i_write_n;
			
			counter_reset = 1'b1;
			
			if (i_addr == 16'h4015 && i_write_n == 1'b0) begin
				nextstate = S_DMA_READ;
			end
		end
		
		S_DMA_READ: begin
			o_stall_cpu = 1'b1;
			
			o_addr = dma_addr;
			o_write_n = 1'b1;
			nextstate = S_DMA_WRITE;
		end
		
		S_DMA_WRITE: begin
			o_stall_cpu = 1'b1;
			
			o_addr = 16'h2004;
			o_wr_data = dma_data;
			o_write_n = 1'b0;
			
			if (counter == 8'd0) begin
				nextstate = S_PASSTHRU;
			end
			else begin
				nextstate = S_DMA_READ;
				counter_enable = 1'b1;
			end
		end
	endcase
end

endmodule

	
	