`timescale 1ns / 1ps

module vid_out
(
	input clk25,
	input clk5,
	
	input [5:0] pixel_colour,
	input [1:0] colour_emphasis,
	input reset,
	input write_pixel,
	input [7:0] pixel_x,
	input [7:0] pixel_y,
	
	output VGA_VSYNCH,
	output VGA_HSYNCH,
	output VGA_OUT_BLANK_Z,
	output VGA_COMP_SYNCH,
	output VGA_OUT_PIXEL_CLOCK,
	
	output [7:0] VGA_OUT_RED,
	output [7:0] VGA_OUT_GREEN,
	output [7:0] VGA_OUT_BLUE
);

wire [23:0] pixel_rgb;
wire [5:0] dout;
reg [9:0] x;
reg [9:0] y;

wire hsync, vsync, blank_z;

//TODO: Emphasis bits

//Framebuffer
fb frame_buff(
	.i_clkA(clk5),
	.i_dataA(pixel_colour),
	.i_writeA(write_pixel),
	.i_addrA({pixel_y, pixel_x}),
	
	.i_clkB(clk25),
	.o_dataB(dout),
	.i_addrB(blank_z ? {y[8:1],x[8:1]} : 16'b0)
);

//palette lookup table

pal_lookup lookup_table(.addr(dout), .clk(clk25), .dout(pixel_rgb));

//delay the control signals until all of the memory reads are done
	
	shift  #
   (
    .LENGTH(4)	
	 ) 
	 vsync_delay
	 (
    .clk(clk25), 
    .sin(vsync), 
    .sout(VGA_VSYNCH), 
    .reset(1'b0)
    );

	shift  #
   (
    .LENGTH(4) 
	 )
	hsync_delay 
	 (
    .clk(clk25), 
    .sin(hsync), 
    .sout(VGA_HSYNCH), 
    .reset(1'b0)
    );

	shift  #
   (
    .LENGTH(3)	
	 )
	 blank_delay
	 (
    .clk(clk25), 
    .sin(blank_z), 
    .sout(VGA_OUT_BLANK_Z), 
    .reset(1'b0)
    );

always @ (posedge clk25 or posedge reset) begin
	if(reset) begin
		x <= 10'd0;
		y <= 10'd0;
	end
	else begin
		if (x < 656)
			x <= x + 10'd1;
		else
		begin
			x <= 10'd0;
			if (y < 502)
				y <= y + 10'd1;
			else
				y <= 10'd0;
		end
	end
end

assign VGA_OUT_PIXEL_CLOCK = ~clk25;
assign hsync = (x >= (10'd512 + 10'd16) && x <= (10'd512 + 10'd72)) ? 1'b0 : 1'b1;
assign vsync = (y >= (10'd502-10'd21) && y <= (10'd502-10'd18)) ? 1'b0 : 1'b1;

assign blank_z = ~(x >= 10'd512 || y >= 10'd480);
assign VGA_COMP_SYNCH = 1'b0;

assign VGA_OUT_RED = pixel_rgb[23:16];
assign VGA_OUT_GREEN = pixel_rgb[15:8];
assign VGA_OUT_BLUE = pixel_rgb[7:0];

endmodule
